

#include "Blep.h"
#include "aci_cmds.h"
#include "aci_evts.h"
#include "Buggy.h"

sbit LED_BOND at GPIOE_ODR.B12;
sbit LED_CONN at GPIOE_ODR.B15;


extern char writeBuff[32];
extern short readBuff[32];
char CONN_STATE = 0;
char NEW_EVT, EVT_STATUS, EVT_OPCODE;

extern sfr sbit Blep_RST;
extern sfr sbit Blep_RDY;
extern sfr sbit Blep_REQ;
extern sfr sbit Blep_ACT;


void Init_MCU() {
   GPIO_Digital_Output(&GPIOE_ODR, _GPIO_PINMASK_7 | _GPIO_PINMASK_8);
   GPIO_Digital_Input(&GPIOE_IDR, _GPIO_PINMASK_10);
   GPIO_Digital_Input(&GPIOA_IDR, _GPIO_PINMASK_2);

   Blep_RST = 0;
   //UART1_Init(56000);

   SPI3_Init_Advanced(_SPI_FPCLK_DIV16, _SPI_MASTER | _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION | _SPI_LSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12);
   Blep_REQ = 1;
   Blep_RST = 1;
   Delay_ms(100);
   Blep_RST = 0;
   Delay_ms(100);
   Blep_RST = 1;

}

void main() {
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_12 | _GPIO_PINMASK_15);
  Delay_ms(200);
  Init_MCU();
  Init_Buggy();
  Blep_Setup();
  Delay_ms(100);
  LED_BOND = 0;
  LED_CONN = 0;
  Start_Buggy();

  while(1) {
     Blep_Bond(0xB4, 0x20);
     Blep_Read();
     LED_BOND = 1;
     Blep_WaitForEvent(ACI_EVT_CONNECTED);                                      //wait host device
     Blep_WaitForEvent(ACI_EVT_PIPE_STATUS);
     Blep_Read();
     CONN_STATE = 1;
     LED_CONN = CONN_STATE;
     
     while(CONN_STATE){
          NEW_EVT = Blep_EventCode();                                           //Wait for new event
          EVT_STATUS = Blep_EventStatus();                                      //Event Status
          EVT_OPCODE = Blep_EventOpcode();                                      //Event Opcode
          if (NEW_EVT == ACI_EVT_DATA_RECEIVED) {                               //Data received notification event
             LED_CONN = ~ LED_CONN;
             OnDataReceived();
         }else{
            switch(NEW_EVT) {

               case ACI_EVT_HW_ERROR:
                 //DebugInfoEvent returns hardware error debug information
                 //This event is sent from nRF8001 to the external application
                 //controller to provide debug information. In case of firmware
                 //failure this event follows the DeviceStartedEvent.
                 if (CONN_STATE) {
                    delay_ms(20);
                 }else{
                    CONN_STATE = 0;
                 }
                 break;
               case ACI_EVT_CMD_RSP:
                 //CommandResponseEvent confirms reception or execution of an ACI command.
                 if (EVT_STATUS > 0x03) { //error
                   CONN_STATE = 0;
                 }
                 break;
               case  ACI_EVT_BOND_STATUS:
                 //BondStatusEvent returns the bonding procedure execution status
                 if (EVT_STATUS != 0) {
                   CONN_STATE = 0;
                 }
                 break;
               case ACI_EVT_CONNECTED:
                 CONN_STATE = 1;
                 LED_CONN = 1;
                 break;
               case ACI_EVT_DISCONNECTED:
                 //DisconnectedEvent indicates the loss of a connection or
                 //disconnected from peer device
                 CONN_STATE = 0;
            }
         }
     }
     Blep_Disconnect();
     LED_CONN = 0;
     LED_BOND = 0;
     //BlueLE_TestModeState(ACI_TEST_MODE_EXIT);
     delay_ms(10);
  
  }
}